// Copyright 2012 Interface3 Ltd. All rights reserved.

var NUMBER_OF_HORIZONTAL_LIGHTS = 7;
var NUMBER_OF_VERTICAL_LIGHTS = 5;
// Can set opacity for these types "ink", "shape", "image", "text", "line". "group" is a complex type that may be suitable if made up of suitable objects
const SIMPLE_ALLOWABLE_TYPES = ["ink", "calligraphicInk", "shape", "image", "text", "line", "straightLine", "curvedLine", "ellipse", "metafile"];
var FADE_SPEED = 200;
var FADE_OUT_LEVEL = 0.1;
var MAX_OBJECTS_TO_CYCLE_THROUGH = 15;
var MIN_OBJECTS_TO_CYCLE_THROUGH = 10;
var objectsAlreadyChosen = [];
var running = false;
var horizontalStripWidth;
var horizontalStripHeight;
var verticalStripWidth;
var verticalStripHeight;
var stripTop;
var stripLeft;
var stripBottom;
var stripRight;
var objectsAnimationQueue = $({});
var lightsAnimationQueue = $({});
var noRepeat = true;

NB.ready(function() {

    try {
        NB.settings.enableTransparentColorMask(255, 255, 254);
    } catch(error) {
        console.log("Cannot set transparency as not running in notebook");
    }

    createLights();
        
    // size and align the light strips
    horizontalStripWidth = ($('#go').width() * 0.91);
    stripTop = parseInt($('#go').css('top').slice(0, -2));
    stripBottom = parseInt($('#go').css('top').slice(0, -2)) + ($('#go').height() * 0.87);
    
    stripLeft = (($('#go').css('left').slice(0, -2)) * 1);
    stripRight = stripLeft + horizontalStripWidth;
    $('#topStripOfLights,.verticalStripOfLights').css('top', stripTop + 'px');
    $('#bottomStripOfLights').css('top', stripBottom + 'px');
    $('.horizontalStripOfLights').css('left', stripLeft + 'px');
    $('.horizontalStripOfLights').css('width', horizontalStripWidth + 'px');
    horizontalStripHeight = ($('.horizontalLight').width());
    $('.verticalLight').css('height', horizontalStripHeight);
    verticalStripWidth = horizontalStripHeight;
    $('#leftStripOfLights').css('left', (stripLeft - horizontalStripHeight) + 'px');
    $('#rightStripOfLights').css('left', (stripRight + horizontalStripHeight) + 'px');
    $('.horizontalStripOfLights').css('height', horizontalStripHeight + 'px');
    $('.verticalStripOfLights').css('width', verticalStripWidth + 'px');
    verticalStripHeight = (stripBottom - stripTop + $('.horizontalLight').height());
    $('.verticalStripOfLights').css('height', verticalStripHeight);
    
    realignLights('horizontal', $('#topStripOfLights'));
    realignLights('horizontal', $('#bottomStripOfLights'));
    realignLights('vertical', $('#leftStripOfLights'));
    realignLights('vertical', $('#rightStripOfLights'));


    $('#go').click(function() {
    
        if(running == false) {
            
            var objectsOnPage = getAppropriateObjectsOnPage(); // an array of objects
            
            if(objectsOnPage.length > 0) {
            
                running = true;
                $('#infoButton,#memory,#closeButton').fadeOut();
                fadeOutAll(objectsOnPage);
                // if we've been through all the objects then reset and allow previously selected objects back in
                // note that the last object is chosen in a round all of its own
                if(objectsOnPage.length - objectsAlreadyChosen.length <= 0) {
                    objectsAlreadyChosen = []; // clear memory
                }
                if(noRepeat == true) {
                    // remove previously chosen objects from the list
                    for(var previouslyChosenObjectIndex = 0; previouslyChosenObjectIndex < objectsAlreadyChosen.length; previouslyChosenObjectIndex++) {
                        var indexOfPreviouslyChosenObjectInAllObjects = objectsOnPage.indexOfGUIDMatch(objectsAlreadyChosen[previouslyChosenObjectIndex]);
                        if(indexOfPreviouslyChosenObjectInAllObjects != -1) {
                            objectsOnPage.remove(indexOfPreviouslyChosenObjectInAllObjects);
                        }
                    }
                }
                chooseObjectAtRandom(objectsOnPage);
            }
        }
    });
    
    $('#memory').click(function() {
        if(running == false) {
            objectsAlreadyChosen = []; // clear memory
            if($("#memory").css('opacity') == 1) {
                $("#memory").fadeTo('fast', 0.3);
                noRepeat = false;
            } else {
                $("#memory").fadeTo('fast', 1);
                noRepeat = true;
            }
            saveNoRepeatSettings();
        }
    });
    
    $('#closeButton').click(function() {
        allObjectsToFullVisibility();
        $('body').fadeOut(250, function() {
                NB.page.deleteObject(NB.getObject(NB.HOSTGUID))
            }
        );
    });
    
    $('#infoButton').click(function() {
        if(running == false) {
            $('#go,#memory,.light,#infoButton,#closeButton').fadeOut('fast').delay(5000).fadeIn('fast');
            $('#infoOverlay').fadeIn('fast').delay(5000).fadeOut('fast');
        }
    });
    
    loadSettings();
   
});

function savePreviousObjectsSettings() {
    try {
        details = {};
        details['previousObjects'] = objectsAlreadyChosen;
        printObjectsAlreadyChosen();
        NB.persist.save("randomObjectSelector-previousObjects", details);
    } catch(error) {
        console.log("Failed to save settings because notebook is not running");
    }
}

function printObjectsAlreadyChosen() {
    console.log("printObjectsAlreadyChosen");
    console.log(objectsAlreadyChosen);
}

function saveNoRepeatSettings() {
    try {
        details = {};
        details['noRepeat'] = noRepeat;
        NB.persist.save("randomObjectSelector-noRepeat", details);
    } catch(error) {
        console.log("Failed to save settings because notebook is not running");
    }
}

function loadSettings() {
    var loadResults = NB.persist.load('randomObjectSelector-noRepeat')
    if(loadResults != null) {
        var settings = JSON.parse(loadResults);
        noRepeat = settings.noRepeat;
        if(noRepeat == true) {
            $("#memory").fadeTo('fast', 1);
        } else {
            $("#memory").fadeTo('fast', 0.3);
        }
    }
    
    loadResults = NB.persist.load('randomObjectSelector-previousObjects')
    if(loadResults != null) {
        var settings = JSON.parse(loadResults);
        objectsAlreadyChosen = settings.previousObjects;
        printObjectsAlreadyChosen();
    }
}

function allObjectsToFullVisibility() {
    var allObjects = getAppropriateObjectsOnPage();
    fadeInAll(allObjects);
}

Array.prototype.indexOfGUIDMatch = function(objectWithGuid) {
    for(var index = 0; index < this.length; index++) {
        if(this[index].guid == objectWithGuid.guid) {
            return index;
        }
    }
    return -1;
};


function getAppropriateObjectsOnPage() {
    var objectsThatCanBeHandledOnPage = []; 
    var allObjects = NB.page.getObjectsAsArray();
    for(var objectIndex = 0; objectIndex < allObjects.length; objectIndex++) {
        if(isSuitableObject(allObjects[objectIndex]) || isSuitableGroup(allObjects[objectIndex])) {
            var tmp1 = allObjects[objectIndex].type, tmp2 = allObjects[objectIndex].name;
            console.log(" Adding object " + allObjects[objectIndex].name + " of type " + allObjects[objectIndex].type + " to suitable list");
            objectsThatCanBeHandledOnPage.push(allObjects[objectIndex]);
        }
    }
    console.log(" Added " + objectsThatCanBeHandledOnPage.length + " to the suitable list");
    for (var i = 0; i < objectsThatCanBeHandledOnPage.length; i++) console.log("    " + objectsThatCanBeHandledOnPage[i].name);
    return objectsThatCanBeHandledOnPage;
}

function isSuitableObject(object) {

    if(object.lockType == "Lock In Place") {
        return false;
    }
    if (object.x < 0 || object.y < 0) {
        return;
    }
    
    console.log(" Probing object " + object.name + " of type " + object.type);
    if(SIMPLE_ALLOWABLE_TYPES.indexOf(object.type) != -1) {
        console.log("    " + object.name + " is of supported type [" + object.type + "]");
        return true;
    } else if(isSuitableGroup(object)) {
        console.log("    " + object.name + " is a group which contains object of supported type");
        return true;
    }
    return false;
}

function isSuitableGroup(object) {

    if(object.lockType == "Lock In Place") {
        return false;
    }
    if (object.x < 0 || object.y < 0) {
        return;
    }

    if(object.type == "group") {
        var children = object.getChildren();
        for(var childIndex = 0; childIndex < children.length; childIndex++) {
            console.log("    " + "Probing group component " + children[childIndex].name + " of type " + children[childIndex].type);
            if(!isSuitableObject(children[childIndex])) {
                return false;
            }
        }
        return true;
    }
    return false;
}
 

function realignLights(direction, holder) {
    var lights = $(holder).find('.' + direction + 'Light');
    var holderSize = $(holder).height() - $(lights[0]).height();
    var topLeft = 'top';
    if(direction == 'horizontal') {
        holderSize = $(holder).width();
        topLeft = 'left';
    }
    var spacing = holderSize / (lights.length - 1);

    for(var lightIndex = 0; lightIndex < lights.length; lightIndex++) {
        $(lights[lightIndex]).css(topLeft, (lightIndex * spacing) + 'px');
    }
}

function createLights() {
    var topStripOfLights = $('<div/>', {'id': 'topStripOfLights', 'class': 'horizontalStripOfLights'});
    topStripOfLights.css('left', stripLeft);
    topStripOfLights.css('top', stripTop);
    topStripOfLights.insertAfter('#memory');
    
    var bottomStripOfLights = $('<div/>', {'id': 'bottomStripOfLights', 'class': 'horizontalStripOfLights'});
    bottomStripOfLights.css('left', stripLeft);
    bottomStripOfLights.css('top', stripTop);
    bottomStripOfLights.insertAfter('#memory');
    
    var leftStripOfLights = $('<div/>', {'id': 'leftStripOfLights', 'class': 'verticalStripOfLights'});
    leftStripOfLights.css('left', stripLeft);
    leftStripOfLights.css('top', stripTop);
    leftStripOfLights.insertAfter('#memory');
    
    var rightStripOfLights = $('<div/>', {'id': 'rightStripOfLights', 'class': 'verticalStripOfLights'});
    rightStripOfLights.css('left', stripLeft);
    rightStripOfLights.css('top', stripTop);
    rightStripOfLights.insertAfter('#memory');
    
    var lightOrder = 0;
    lightOrder = createStripOfLights('horizontal', $('#topStripOfLights'), lightOrder, "normalOrder");
    lightOrder = createStripOfLights('vertical', $('#rightStripOfLights'), lightOrder, "normalOrder");
    lightOrder = createStripOfLights('horizontal', $('#bottomStripOfLights'), lightOrder, "reverseOrder");
    createStripOfLights('vertical', $('#leftStripOfLights'), lightOrder, "reverseOrder");
}

function createStripOfLights(direction, holder, lightOrder, order) {
    var numberOfLights = NUMBER_OF_HORIZONTAL_LIGHTS;
    if(direction == 'vertical') {
        numberOfLights = NUMBER_OF_VERTICAL_LIGHTS;
    }

    for(var lightIndex = 0; lightIndex < numberOfLights; lightIndex++) {
        var correctedLightOrder = lightOrder;
        if(order == "reverseOrder") {
            correctedLightOrder = lightOrder + numberOfLights - (lightIndex * 2);
        }
        createLight(direction, holder, correctedLightOrder, order);
        lightOrder++;
    }
    return lightOrder;
}

function createLight(direction, holder, lightOrder) {
    var newLight = $('<div/>', {'class': direction + 'Light light', 'order': lightOrder});
    $(newLight).data('lightOrder', lightOrder);
    newLight.appendTo(holder);
}

function fadeOutAll(objects) {
    for (var objectIndex = 0; objectIndex < objects.length; objectIndex++) {
        fadeOut(objects[objectIndex]);
    }
}

function fadeInAll(objects) {
    for (var objectIndex = 0; objectIndex < objects.length; objectIndex++) {
        fadeIn(objects[objectIndex]);
    }
}

function animateLight(object) {
    lightsAnimationQueue.queue('lightsAnimation', function(next) {
        $(object).animate({opacity: 0}, 50, function() {
            $(object).animate({opacity: 1}, 50);
            if(lightsAnimationQueue.queue('lightsAnimation').length == 0 && running == true) {
                // stop lights animations
                lightsAnimationQueue.clearQueue('lightsAnimation');
                lightsAnimationQueue.stop();
                animateLights();
                lightsAnimationQueue.dequeue('lightsAnimation');
            } else {
                next();
            }
        });
    });
}

function animateLights() {
    var lights = $('.light');
    lights.sort(lightsSort);
    for(var lightIndex = 0; lightIndex < lights.length; lightIndex++) {
        animateLight(lights[lightIndex]);
    }
}

function flashAllLights() {
    // stop lights animations
    lightsAnimationQueue.clearQueue('lightsAnimation');
    var lights = $('.light');
    for(var lightIndex = 0; lightIndex < lights.length; lightIndex++) {
        $(lights[lightIndex]).transit({scale: 1.4, opacity: 1, easing: $.easing.easeOutExpo}, 400,
            function() { // on complete
                $(this).transit({scale: 1, duration: 300, easing: $.easing.easeInExpo});
            }
        );
    }
}

function lightsSort(a, b) {
    return $(a).data('lightOrder') > $(b).data('lightOrder') ?1 :-1;
}

function chooseObjectAtRandom(objects) {
    //var rounds = Math.max(Math.round((Math.random() * (MAX_ROUNDS_TOTAL - MIN_ROUNDS_TOTAL)) + MIN_ROUNDS_TOTAL), (MIN_ROUNDS_PER_OBJECT * objects.length) + Math.round(Math.random() * (MAX_EXTRA_ROUNDS * objects.length)));

    var numberOfObjectsToCycleThrough = Math.round((Math.random() * (MAX_OBJECTS_TO_CYCLE_THROUGH - MIN_OBJECTS_TO_CYCLE_THROUGH)) + MIN_OBJECTS_TO_CYCLE_THROUGH);

    if(objects.length == 1) {
        numberOfObjectsToCycleThrough = 2;
    }
    
    if(objects.length == 2) {
        numberOfObjectsToCycleThrough = Math.round(numberOfObjectsToCycleThrough / 2);
    }
    
    if(objects.length == 3) {
        numberOfObjectsToCycleThrough = Math.round(numberOfObjectsToCycleThrough / 1.5);
    }
    
    
    // collect random objects from the set (leave them in so they can be selected again if needed)
    var objectsToCycleThrough = new Array();
    for(var objectIndex = 0; objectIndex < numberOfObjectsToCycleThrough; objectIndex++) {
        objectsToCycleThrough.push(Math.round(Math.random() * (objects.length - 1)));
    }

    animateLights();
        
    for (var objectIndex = 0; objectIndex < objectsToCycleThrough.length; objectIndex++) {
        if(objectIndex == objectsToCycleThrough.length - 1) {
            // select object here by fading in - not in and out again
            objectsAlreadyChosen.push(objects[objectsToCycleThrough[objectIndex]]);
            savePreviousObjectsSettings();
            fadeInOut(objects[objectsToCycleThrough[objectIndex]], true, objectsToCycleThrough[objectIndex]);
            lightsAnimationQueue.dequeue('lightsAnimation');
            objectsAnimationQueue.dequeue(); // start the animations
        } else {
            fadeInOut(objects[objectsToCycleThrough[objectIndex]], false, objectsToCycleThrough[objectIndex]);
        }
        
    }
    
}

function fadeOut(object) {
    $(object).animate({transparency: FADE_OUT_LEVEL}, {
        duration: FADE_SPEED
    });
}

function fadeIn(object) {
    $(object).animate({transparency: 1}, {
        duration: 200
    });
}

function fadeInOut(object, selectObject, objectIndex) {

    if(selectObject) {
        $(objectsAnimationQueue).queue(function(next) {
            $(object).animate({transparency: 1}, {
                duration: FADE_SPEED,
                complete: function() { 
                    running = false;
                    $('#infoButton,#memory,#closeButton').fadeIn();
                    object.moveToFront();
                    flashAllLights();
                    next();
                }
            });
        });
        
    } else {
    
        $(objectsAnimationQueue).queue(function(next) {
            $(object).animate({transparency: 1}, {
                duration: FADE_SPEED,
                complete: function() { next(); }
            });
        });
    
        $(objectsAnimationQueue).queue(function(next) {
            $(object).animate({transparency: FADE_OUT_LEVEL}, {
                duration: FADE_SPEED,
                complete: function() { next(); }
            });
        });
    }    
}

// Array Remove - By John Resig (MIT Licensed) http://ejohn.org/blog/javascript-array-remove/
Array.prototype.remove = function(from, to) {
  var rest = this.slice((to || from) + 1 || this.length);
  this.length = from < 0 ? this.length + from : from;
  return this.push.apply(this, rest);
};
